"""
Validator for a regular language.
"""
from __future__ import unicode_literals

from prompt_toolkit.document import Document
from prompt_toolkit.validation import ValidationError, Validator

from .compiler import _CompiledGrammar

__all__ = [
    'GrammarValidator',
]


class GrammarValidator(Validator):
    """
    Validator which can be used for validation according to variables in
    the grammar. Each variable can have its own validator.

    :param compiled_grammar: `GrammarCompleter` instance.
    :param validators: `dict` mapping variable names of the grammar to the
                       `Validator` instances to be used for each variable.
    """
    def __init__(self, compiled_grammar, validators):
        assert isinstance(compiled_grammar, _CompiledGrammar)
        assert isinstance(validators, dict)

        self.compiled_grammar = compiled_grammar
        self.validators = validators

    def validate(self, document):
        # Parse input document.
        # We use `match`, not `match_prefix`, because for validation, we want
        # the actual, unambiguous interpretation of the input.
        m = self.compiled_grammar.match(document.text)

        if m:
            for v in m.variables():
                validator = self.validators.get(v.varname)

                if validator:
                    # Unescape text.
                    unwrapped_text = self.compiled_grammar.unescape(v.varname, v.value)

                    # Create a document, for the completions API (text/cursor_position)
                    inner_document = Document(unwrapped_text, len(unwrapped_text))

                    try:
                        validator.validate(inner_document)
                    except ValidationError as e:
                        raise ValidationError(
                            cursor_position=v.start + e.cursor_position,
                            message=e.message)
        else:
            raise ValidationError(cursor_position=len(document.text),
                                  message='Invalid command')



from __future__ import unicode_literals

from .base import DummyInput, Input
from .defaults import create_input, get_default_input, set_default_input

__all__ = [
    # Base.
    'Input',
    'DummyInput',

    # Defaults.
    'create_input',
    'get_default_input',
    'set_default_input',
]



"""
Styling for prompt_toolkit applications.
"""
from __future__ import absolute_import, unicode_literals

from .base import (
    ANSI_COLOR_NAMES,
    DEFAULT_ATTRS,
    Attrs,
    BaseStyle,
    DummyStyle,
    DynamicStyle,
)
from .defaults import default_pygments_style, default_ui_style
from .named_colors import NAMED_COLORS
from .pygments import (
    pygments_token_to_classname,
    style_from_pygments_cls,
    style_from_pygments_dict,
)
from .style import Priority, Style, merge_styles, parse_color
from .style_transformation import (
    AdjustBrightnessStyleTransformation,
    ConditionalStyleTransformation,
    DummyStyleTransformation,
    DynamicStyleTransformation,
    ReverseStyleTransformation,
    SetDefaultColorStyleTransformation,
    StyleTransformation,
    SwapLightAndDarkStyleTransformation,
    merge_style_transformations,
)

__all__ = [
    # Base.
    'Attrs',
    'DEFAULT_ATTRS',
    'ANSI_COLOR_NAMES',
    'BaseStyle',
    'DummyStyle',
    'DynamicStyle',

    # Defaults.
    'default_ui_style',
    'default_pygments_style',

    # Style.
    'Style',
    'Priority',
    'merge_styles',
    'parse_color',

    # Style transformation.
    'StyleTransformation',
    'SwapLightAndDarkStyleTransformation',
    'ReverseStyleTransformation',
    'SetDefaultColorStyleTransformation',
    'AdjustBrightnessStyleTransformation',
    'DummyStyleTransformation',
    'ConditionalStyleTransformation',
    'DynamicStyleTransformation',
    'merge_style_transformations',

    # Pygments.
    'style_from_pygments_cls',
    'style_from_pygments_dict',
    'pygments_token_to_classname',

    # Named colors.
    'NAMED_COLORS',
]



from __future__ import unicode_literals

from .base import ProgressBar
from .formatters import (
    Bar,
    Formatter,
    IterationsPerSecond,
    Label,
    Percentage,
    Progress,
    Rainbow,
    SpinningWheel,
    Text,
    TimeElapsed,
    TimeLeft,
)

__all__ = [
    'ProgressBar',

    # Formatters.
    'Formatter',
    'Text',
    'Label',
    'Percentage',
    'Bar',
    'Progress',
    'TimeElapsed',
    'TimeLeft',
    'IterationsPerSecond',
    'SpinningWheel',
    'Rainbow',
]



from __future__ import unicode_literals

from .key_bindings import (
    ConditionalKeyBindings,
    DynamicKeyBindings,
    KeyBindings,
    KeyBindingsBase,
    merge_key_bindings,
)

__all__ = [
    'KeyBindingsBase',
    'KeyBindings',
    'ConditionalKeyBindings',
    'merge_key_bindings',
    'DynamicKeyBindings',
]



from __future__ import unicode_literals

from .base import DummyOutput, Output
from .color_depth import ColorDepth
from .defaults import create_output, get_default_output, set_default_output

__all__ = [
    # Base.
    'Output',
    'DummyOutput',

    # Color depth.
    'ColorDepth',

    # Defaults.
    'create_output',
    'get_default_output',
    'set_default_output',
]



"""
Clipboard for command line interface.
"""
from __future__ import unicode_literals

from abc import ABCMeta, abstractmethod

import six
from six import with_metaclass

from prompt_toolkit.selection import SelectionType

__all__ = [
    'Clipboard',
    'ClipboardData',
    'DummyClipboard',
    'DynamicClipboard',
]


class ClipboardData(object):
    """
    Text on the clipboard.

    :param text: string
    :param type: :class:`~prompt_toolkit.selection.SelectionType`
    """
    def __init__(self, text='', type=SelectionType.CHARACTERS):
        assert isinstance(text, six.string_types)
        assert type in (SelectionType.CHARACTERS, SelectionType.LINES, SelectionType.BLOCK)

        self.text = text
        self.type = type


class Clipboard(with_metaclass(ABCMeta, object)):
    """
    Abstract baseclass for clipboards.
    (An implementation can be in memory, it can share the X11 or Windows
    keyboard, or can be persistent.)
    """
    @abstractmethod
    def set_data(self, data):
        """
        Set data to the clipboard.

        :param data: :class:`~.ClipboardData` instance.
        """

    def set_text(self, text):  # Not abstract.
        """
        Shortcut for setting plain text on clipboard.
        """
        assert isinstance(text, six.string_types)
        self.set_data(ClipboardData(text))

    def rotate(self):
        """
        For Emacs mode, rotate the kill ring.
        """

    @abstractmethod
    def get_data(self):
        """
        Return clipboard data.
        """


class DummyClipboard(Clipboard):
    """
    Clipboard implementation that doesn't remember anything.
    """
    def set_data(self, data):
        pass

    def set_text(self, text):
        pass

    def rotate(self):
        pass

    def get_data(self):
        return ClipboardData()


class DynamicClipboard(Clipboard):
    """
    Clipboard class that can dynamically returns any Clipboard.

    :param get_clipboard: Callable that returns a :class:`.Clipboard` instance.
    """
    def __init__(self, get_clipboard):
        assert callable(get_clipboard)
        self.get_clipboard = get_clipboard

    def _clipboard(self):
        clipboard = self.get_clipboard() or DummyClipboard()
        assert isinstance(clipboard, Clipboard)
        return clipboard

    def set_data(self, data):
        self._clipboard().set_data(data)

    def set_text(self, text):
        self._clipboard().set_text(text)

    def rotate(self):
        self._clipboard().rotate()

    def get_data(self):
        return self._clipboard().get_data()



"""
Collection of reusable components for building full screen applications.
These are higher level abstractions on top of the `prompt_toolkit.layout`
module.

Most of these widgets implement the ``__pt_container__`` method, which makes it
possible to embed these in the layout like any other container.
"""
from __future__ import unicode_literals

from .base import (
    Box,
    Button,
    Checkbox,
    Frame,
    HorizontalLine,
    Label,
    ProgressBar,
    RadioList,
    Shadow,
    TextArea,
    VerticalLine,
)
from .dialogs import Dialog
from .menus import MenuContainer, MenuItem
from .toolbars import (
    ArgToolbar,
    CompletionsToolbar,
    FormattedTextToolbar,
    SearchToolbar,
    SystemToolbar,
    ValidationToolbar,
)

__all__ = [
    # Base.
    'TextArea',
    'Label',
    'Button',
    'Frame',
    'Shadow',
    'Box',
    'VerticalLine',
    'HorizontalLine',
    'RadioList',
    'Checkbox',
    'ProgressBar',

    # Toolbars.
    'ArgToolbar',
    'CompletionsToolbar',
    'FormattedTextToolbar',
    'SearchToolbar',
    'SystemToolbar',
    'ValidationToolbar',

    # Dialogs.
    'Dialog',

    # Menus.
    'MenuContainer',
    'MenuItem',
]



"""
Lexer interface and implementations.
Used for syntax highlighting.
"""
from __future__ import unicode_literals

from .base import DynamicLexer, Lexer, SimpleLexer
from .pygments import PygmentsLexer, RegexSync, SyncFromStart, SyntaxSync

__all__ = [
    # Base.
    'Lexer',
    'SimpleLexer',
    'DynamicLexer',

    # Pygments.
    'PygmentsLexer',
    'RegexSync',
    'SyncFromStart',
    'SyntaxSync',
]



from __future__ import unicode_literals

from .dialogs import (
    button_dialog,
    input_dialog,
    message_dialog,
    progress_dialog,
    radiolist_dialog,
    yes_no_dialog,
)
from .progress_bar import ProgressBar
from .prompt import (
    CompleteStyle,
    PromptSession,
    confirm,
    create_confirm_session,
    prompt,
)
from .utils import (
    clear,
    clear_title,
    print_container,
    print_formatted_text,
    set_title,
)

__all__ = [
    # Dialogs.
    'input_dialog',
    'message_dialog',
    'progress_dialog',
    'radiolist_dialog',
    'yes_no_dialog',
    'button_dialog',

    # Prompts.
    'PromptSession',
    'prompt',
    'confirm',
    'create_confirm_session',
    'CompleteStyle',

    # Progress bars.
    'ProgressBar',

    # Utils.
    'clear',
    'clear_title',
    'print_formatted_text',
    'set_title',
]



from __future__ import unicode_literals

from .base import Always, Filter, Never

__all__ = [
    'to_filter',
    'is_true',
]

_always = Always()
_never = Never()


def to_filter(bool_or_filter):
    """
    Accept both booleans and Filters as input and
    turn it into a Filter.
    """
    if not isinstance(bool_or_filter, (bool, Filter)):
        raise TypeError('Expecting a bool or a Filter instance. Got %r' % bool_or_filter)

    return {
        True: _always,
        False: _never,
    }.get(bool_or_filter, bool_or_filter)


def is_true(value):
    """
    Test whether `value` is True. In case of a Filter, call it.

    :param value: Boolean or `Filter` instance.
    """
    return to_filter(value)()



from __future__ import unicode_literals

from .base import (
    CompleteEvent,
    Completer,
    Completion,
    DummyCompleter,
    DynamicCompleter,
    ThreadedCompleter,
    get_common_complete_suffix,
    merge_completers,
)
from .filesystem import ExecutableCompleter, PathCompleter
from .fuzzy_completer import FuzzyCompleter, FuzzyWordCompleter
from .word_completer import WordCompleter

__all__ = [
    # Base.
    'Completion',
    'Completer',
    'ThreadedCompleter',
    'DummyCompleter',
    'DynamicCompleter',
    'CompleteEvent',
    'merge_completers',
    'get_common_complete_suffix',

    # Filesystem.
    'PathCompleter',
    'ExecutableCompleter',

    # Word completer.
    'WordCompleter',
    'FuzzyCompleter',
    'FuzzyWordCompleter',
]
